/**
 * Modified Serialport Lib to communicate with ESP8266 ESP-01 Module with RealTime FW
 */

#ifndef ESPPORT_H_
#define ESPPORT_H_

// define your hardware here:
#define ESPPORT_UART		USARTC1
#define ESPPORT_RXC_vect    USARTC1_RXC_vect
#define ESPPORT_DRE_vect    USARTC1_DRE_vect
// set pin directions
#define ESPPORT_TXD_TO_OUTPUT (PORTC_DIRSET = PIN7_bm)
#define ESPPORT_RXD_TO_INPUT  (PORTC_DIRCLR = PIN6_bm)

#define ESPPORT_EN_TO_OUTPUT  (PORTB_DIRSET = PIN7_bm)
#define ESPPORT_EN_LOW        (PORTB_OUTCLR = PIN7_bm)
#define ESPPORT_EN_HIGH       (PORTB_OUTSET = PIN7_bm)

#define ESPPORT_RST_TO_OUTPUT (PORTB_DIRSET = PIN5_bm)
#define ESPPORT_RST_LOW       (PORTB_OUTCLR = PIN5_bm)
#define ESPPORT_RST_HIGH      (PORTB_OUTSET = PIN5_bm)


int8_t espport_update(void);


/**
 * @brief This function initializes the USART to 8N1 with any possible Baudrate and enables the RXC interrupt.
 */
uint8_t espport_init(uint32_t baudrate);

/**
 * @brief This function send an array (string).
 *
 * It uses 64 byte buffer. It is possible to overrun the buffer, don't send more than 64 bytes at one Time!
 * Check the buffer space with the espport_checkSpace() function, before you use this function.\n
 * Sending is interrupt driven and don't stop the controller. The function uses only the arrayLength.
 * String end (0x00) would not be recognized!
 *
 * @param *anArray The pointer to the array with the data to send.
 * @param arrayLength The number of bytes to send.
 */
void espport_write(uint8_t *anArray, uint8_t arrayLength);

/**
 * @brief This function check how many bytes are free in the transmit buffer.
 *
 * @return Number of free bytes in the transmit buffer.
 */ 
uint8_t espport_checkSpace(void);

/**
 * @brief This function gives you all received data to the next CR (carriage return) in string format.
 *
 * To get the string, it is necessary that your array is big enough to hold all data to the CR.
 * If is not (maxLength reached, before CR is) this function return 0 and data is lost.
 * The maxLength means all chars in the string including 0x00, so it should equals the array length!
 *
 * @param *anArray The pointer to the array assigned with the data. It have to be big enough to store all data received!
 * @param maxLength The length of the array. Correct value is necessary to prevent array overflow.
 *
 * @return The amount of characters stored in the array including the string end sign 0x00!
 *		   If 0 is returned, no CR is received or the array is to small.
 */
uint8_t espport_getLastString(uint8_t *anArray, uint8_t maxLength);

/**
 * @brief This function is like printf.
 *
 * It uses 64 byte buffer. Your output have to be shorter than 64 chars.
 * It use espport_checkSpace() function and wait for enough space if the buffer is full.\n
 * Sending is interrupt driven and don't stop the controller.
 */
void espport_printf(const char *__fmt, ...);

void espport_print(char* aString);


#endif /* ESPPORT_H_ */