/*
 * NixieATX.c
 *
 * Created: 01.10.2019 21:43:29
 * Author : Robert
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdbool.h>
#include <stddef.h>
#include <avr/pgmspace.h>

#include "Clockconfig.h"
#include "Serialport.h"
#include "ESP_RealTime.h"
#include "DS1307.h"
#include "tube_ctrl.h"
#include "ws2812.h"
#include "pir.h"

#define VERSION "1.02d"





// update the tube illumination leds
// light_on = true  -> illumination should be on
// light_on = false -> illumination should be off
// ht_on    = true  -> hours ten tube should be illuminated
// ht_on    = false -> hours ten tube should not be illuminated
void led_illumination_update(bool light_on, bool ht_on)
{
    uint8_t r=100, g=10, b=0;   // define color here
    uint16_t brightness = 20;   // brightness in % of colon led (must be darker than other leds)
    uint8_t i;
    
    if (light_on)
    {
        for (i=0; i<WS2812_LEDCNT; i++)
        {
            if (i == 2)
            {
                WS2812_set_color(((uint16_t)r*brightness)/100, ((uint16_t)g*brightness)/100, ((uint16_t)b*brightness)/100, i);
            }
            else if (i == 0 && !ht_on) // turn hours ten tube of
            {
                WS2812_set_color(0, 0, 0, i);
            }
            else
            {
                WS2812_set_color(r, g, b, i);
            }
        }
        WS2812_update();
    } 
    else
    {
        for (i=0; i<WS2812_LEDCNT; i++)
        {
            WS2812_set_color(0, 0, 0, i);
        }
        WS2812_update();
    }
}


int main(void)
{
    uint8_t retval = 0;
    //uint8_t i;
    
    rtc_time_t uhrzeit;
    //rtc_date_t datum;
    
    //char tag[3] = "Er";
    
    uint8_t last_second = 60;
    uint8_t last_minute = 60;
        
    bool presence_merker = false;
    bool ht_on_merker = false;
    
    clk_init();
    serialport_init(115200);
    espport_init(115200);
    pir_init(true);
    tube_init();
    WS2812_init();

    PMIC_CTRL |= PMIC_HILVLEN_bm|PMIC_MEDLVLEN_bm|PMIC_LOLVLEN_bm;
    sei();
    
    serialport_print("\rNixieATX V: ");
    serialport_print(VERSION);
    serialport_print("\r");
    
    // tube_test();
    
    
    
    retval = DS1307_init();
    serialport_printf("DS1307 stated: %d\r", retval);
    while (retval != 0) // rtc or sync has a problem 
    {
        retval = espport_update();      // update before showing wrong time
        serialport_printf("Synced with %d\r", retval);        
    }
      
    
    while (1) 
    {        
        rtc_read_time(&uhrzeit);
        
        if (uhrzeit.seconds != last_second)
        {
            if (presence_merker)    // if tubes are on
            {
                tube_toggle_colon();    // toggle colon every second
                serialport_print("colon toggled\r");
            } 
            else
            {
                tube_set_colon(false);  // disable colon, necessary if LED colon powered from +5V instead of HV
                serialport_print("colon off\r");
            }
            //rtc_read_date(&datum);           
            //serialport_printf("RTC: %2d:%02d:%02d - %s %d.%d.%d\r",
                //uhrzeit.hours,
                //uhrzeit.minutes,
                //uhrzeit.seconds,
                //rtc_getStrWeekday(tag, datum.weekday),
                //datum.day,
                //datum.month,
                //datum.year);
            last_second = uhrzeit.seconds;
        }
        
        if (uhrzeit.minutes != last_minute)
        {
            tube_show_time(uhrzeit.hours, uhrzeit.minutes);
            
            // set timepoints for NTP sync. Choose times with odd minutes for better NTP performance
            if ( ( (uhrzeit.hours ==  3) && (uhrzeit.minutes ==  1) ) ||
                 ( (uhrzeit.hours ==  9) && (uhrzeit.minutes == 17) ) ||
                 ( (uhrzeit.hours == 15) && (uhrzeit.minutes ==  9) ) ||
                 ( (uhrzeit.hours == 19) && (uhrzeit.minutes == 18) ) ||
                 ( (uhrzeit.hours == 21) && (uhrzeit.minutes == 11) ) )
            {
                tube_set_colon(false);                  // turn colon off during sync
                retval = espport_update();
                serialport_printf("Synced with %d\r", retval);
                rtc_read_time(&uhrzeit);
                tube_set_colon(true);
            }
            
            serialport_printf("%02d:%02d:%02d\r",uhrzeit.hours, uhrzeit.minutes, uhrzeit.seconds);
            last_minute = uhrzeit.minutes;
        }   
        
        if (tube_ht_on())
        {
            if (ht_on_merker == false) // hours tens changed from off to on
            {
                ht_on_merker = true;
                led_illumination_update(presence_merker, ht_on_merker);
            }
        } 
        else
        {
            if (ht_on_merker == true) // hours tens changed from on to off
            {
                ht_on_merker = false;
                led_illumination_update(presence_merker, ht_on_merker);
            }
        }
        
        pir_poll();
        if (pir_presence())
        {
            if (presence_merker == false)   // presence changed from false to true
            {
                presence_merker = true;
                tube_hv(presence_merker);
                led_illumination_update(presence_merker, ht_on_merker);
                tube_set_colon(presence_merker);
                serialport_print("presence changed from false to true\r");
            }
        }
        else
        {
            if (presence_merker == true)    // presence changed from true to false
            {
                presence_merker = false;
                tube_hv(presence_merker);
                led_illumination_update(presence_merker, ht_on_merker);
                tube_set_colon(presence_merker);
                serialport_print("presence changed from true to false\r");
            }
        }
        
        _delay_ms(10);        
    }
}
